package de.hftstuttgart.unifiedticketing.core;

import org.junit.jupiter.api.BeforeEach;
import org.junit.jupiter.api.Test;

import java.util.Arrays;
import java.util.Collections;
import java.util.HashSet;
import java.util.Set;

import static org.junit.jupiter.api.Assertions.*;
import static org.mockito.Mockito.*;

public class TicketBuilderTest
{
    public TicketBuilder instance;

    @BeforeEach
    public void initBeforeEach()
    {
        instance = mock(TicketBuilder.class,
            withSettings().useConstructor(mock(TicketSystem.class)).defaultAnswer(CALLS_REAL_METHODS));
    }

    @Test
    public void testInit()
    {
        TicketSystem ts = mock(TicketSystem.class);
        instance = mock(TicketBuilder.class, withSettings().useConstructor(ts));

        assertAll(
            () -> assertSame(ts, instance.parent),
            () -> assertNull(instance.description),
            () -> assertNull(instance.labels),
            () -> assertNull(instance.title)
            // seems not possible with mock
//            () -> assertThrows(AssertionException.class, () -> new TicketBuilderTestImpl(null))
        );
    }

    @Test
    public void testDescription()
    {
        String expected = "some descriptive stuff";
        instance.description(expected);
        assertEquals(expected, instance.description);
    }

    @Test
    public void testLabelsBySet()
    {
        Set<String> expected = new HashSet<>(Arrays.asList("unifiedticketing", "bug", "To Do"));
        instance.labels(expected);
        assertEquals(expected, instance.labels);
    }

    @Test
    public void testLabelsByArray()
    {
        String[] input = new String[]{"unifiedticketing", "feature", "review"};
        instance.labels(input);
        verify(instance, times(1)).labels(new HashSet<>(Arrays.asList(input)));
    }

    @Test
    public void testTitle()
    {
        String expected = "outstanding title: yolo";
        instance.title(expected);
        assertEquals(expected, instance.title);
    }

    @Test
    public void testFluentApi()
    {
        TicketBuilder before = instance;
        TicketBuilder after = instance
            .description("some")
            .labels(new String[]{"random"})
            .labels(new HashSet<>(Collections.singleton("values")))
            .title("here");

        assertSame(before, after);
    }
}
